@@ -4,20 +4,17 @@ import times
 from collections import namedtuple
 from uuid import uuid4
 from cPickle import loads, dumps, UnpicklingError
-from .connections import get_current_connection
+from .local import LocalStack
+from .connections import resolve_connection
 from .exceptions import UnpickleError, NoSuchJobError
 
 
-JOB_ATTRS = set(['origin', '_func_name', 'ended_at', 'description', '_args',
-                 'created_at', 'enqueued_at', 'connection', '_result', 'result',
-                 'timeout', '_kwargs', 'exc_info', '_id', 'data', '_instance',
-                 'result_ttl', '_status', 'status'])
-
 def enum(name, *sequential, **named):
     values = dict(zip(sequential, range(len(sequential))), **named)
     return type(name, (), values)
 
-Status = enum('Status', QUEUED='queued', FINISHED='finished', FAILED='failed')
+Status = enum('Status', QUEUED='queued', FINISHED='finished', FAILED='failed',
+                        STARTED='started')
 
 
 def unpickle(pickled_string):
@@ -52,6 +49,16 @@ def requeue_job(job_id, connection=None):
     fq.requeue(job_id)
 
 
+def get_current_job():
+    """Returns the Job instance that is currently being executed.  If this
+    function is invoked from outside a job context, None is returned.
+    """
+    job_id = _job_stack.top
+    if job_id is None:
+        return None
+    return Job.fetch(job_id)
+
+
 class Job(object):
     """A Job is just a convenient datastructure to pass around job (meta) data.
     """
@@ -109,6 +116,10 @@ class Job(object):
     def is_failed(self):
         return self.status == Status.FAILED
 
+    @property
+    def is_started(self):
+        return self.status == Status.STARTED
+
     @property
     def func(self):
         func_name = self.func_name
@@ -135,9 +146,9 @@ class Job(object):
         return self._kwargs
 
     @classmethod
-    def exists(cls, job_id):
+    def exists(cls, job_id, connection=None):
         """Returns whether a job hash exists for the given job ID."""
-        conn = get_current_connection()
+        conn = resolve_connection(connection)
         return conn.exists(cls.key_for(job_id))
 
     @classmethod
@@ -149,10 +160,17 @@ class Job(object):
         job.refresh()
         return job
 
+    @classmethod
+    def safe_fetch(cls, id, connection=None):
+        """Fetches a persisted job from its corresponding Redis key, but does
+        not instantiate it, making it impossible to get UnpickleErrors.
+        """
+        job = cls(id, connection=connection)
+        job.refresh(safe=True)
+        return job
+
     def __init__(self, id=None, connection=None):
-        if connection is None:
-            connection = get_current_connection()
-        self.connection = connection
+        self.connection = resolve_connection(connection)
         self._id = id
         self.created_at = times.now()
         self._func_name = None
@@ -168,6 +186,7 @@ class Job(object):
         self.timeout = None
         self.result_ttl = None
         self._status = None
+        self.meta = {}
 
 
     # Data access
@@ -230,7 +249,7 @@ class Job(object):
 
 
     # Persistence
-    def refresh(self):  # noqa
+    def refresh(self, safe=False):  # noqa
         """Overwrite the current instance's properties with the values in the
         corresponding Redis key.
 
@@ -247,7 +266,12 @@ class Job(object):
             else:
                 return times.to_universal(date_str)
 
-        self._func_name, self._instance, self._args, self._kwargs = unpickle(obj.get('data'))  # noqa
+        self.data = obj.get('data')
+        try:
+            self._func_name, self._instance, self._args, self._kwargs = unpickle(self.data)
+        except UnpickleError:
+            if not safe:
+                raise
         self.created_at = to_date(obj.get('created_at'))
         self.origin = obj.get('origin')
         self.description = obj.get('description')
@@ -257,12 +281,8 @@ class Job(object):
         self.exc_info = obj.get('exc_info')
         self.timeout = int(obj.get('timeout')) if obj.get('timeout') else None
         self.result_ttl = int(obj.get('result_ttl')) if obj.get('result_ttl') else None # noqa
-        self._status = obj.get('status') if obj.get('status') else None # noqa
-
-        # Overwrite job's additional attrs (those not in JOB_ATTRS), if any
-        additional_attrs = set(obj.keys()).difference(JOB_ATTRS)
-        for attr in additional_attrs:
-            setattr(self, attr, obj[attr])
+        self._status = obj.get('status') if obj.get('status') else None
+        self.meta = unpickle(obj.get('meta')) if obj.get('meta') else {}
 
     def save(self):
         """Persists the current job instance to its corresponding Redis key."""
@@ -291,18 +311,9 @@ class Job(object):
             obj['result_ttl'] = self.result_ttl
         if self._status is not None:
             obj['status'] = self._status
-        """
-        Store additional attributes from job instance into Redis. This is done
-        so that third party libraries using RQ can store additional data
-        directly on ``Job`` instances. For example:
+        if self.meta:
+            obj['meta'] = dumps(self.meta)
 
-        job = Job.create(func)
-        job.foo = 'bar'
-        job.save() # Will persist the 'foo' attribute
-        """
-        additional_attrs = set(self.__dict__.keys()).difference(JOB_ATTRS)
-        for attr in additional_attrs:
-            obj[attr] = getattr(self, attr)
         self.connection.hmset(key, obj)
 
     def cancel(self):
@@ -323,9 +334,12 @@ class Job(object):
 
     # Job execution
     def perform(self):  # noqa
-        """Invokes the job function with the job arguments.
-        """
-        self._result = self.func(*self.args, **self.kwargs)
+        """Invokes the job function with the job arguments."""
+        _job_stack.push(self.id)
+        try:
+            self._result = self.func(*self.args, **self.kwargs)
+        finally:
+            assert self.id == _job_stack.pop()
         return self._result
 
 
@@ -352,3 +366,46 @@ class Job(object):
 
     def __hash__(self):
         return hash(self.id)
+
+
+    # Backwards compatibility for custom properties
+    def __getattr__(self, name):  # noqa
+        import warnings
+        warnings.warn(
+                "Getting custom properties from the job instance directly "
+                "will be unsupported as of RQ 0.4. Please use the meta dict "
+                "to store all custom variables.  So instead of this:\n\n"
+                "\tjob.foo\n\n"
+                "Use this:\n\n"
+                "\tjob.meta['foo']\n",
+                SyntaxWarning)
+        try:
+            return self.__dict__['meta'][name]  # avoid recursion
+        except KeyError:
+            return getattr(super(Job, self), name)
+
+    def __setattr__(self, name, value):
+        # Ignore the "private" fields
+        private_attrs = set(['origin', '_func_name', 'ended_at',
+            'description', '_args', 'created_at', 'enqueued_at', 'connection',
+            '_result', 'result', 'timeout', '_kwargs', 'exc_info', '_id',
+            'data', '_instance', 'result_ttl', '_status', 'status', 'meta'])
+
+        if name in private_attrs:
+            object.__setattr__(self, name, value)
+            return
+
+        import warnings
+        warnings.warn(
+                "Setting custom properties on the job instance directly will "
+                "be unsupported as of RQ 0.4. Please use the meta dict to "
+                "store all custom variables.  So instead of this:\n\n"
+                "\tjob.foo = 'bar'\n\n"
+                "Use this:\n\n"
+                "\tjob.meta['foo'] = 'bar'\n",
+                SyntaxWarning)
+
+        self.__dict__['meta'][name] = value
+
+
+_job_stack = LocalStack()
