@@ -11,16 +11,13 @@ except ImportError:
 import socket
 import signal
 import traceback
+import logging
 from cPickle import dumps
-try:
-    from logbook import Logger
-    Logger = Logger   # Does nothing except it shuts up pyflakes annoying error
-except ImportError:
-    from logging import Logger
 from .queue import Queue, get_failed_queue
 from .connections import get_current_connection
-from .job import Status
+from .job import Job, Status
 from .utils import make_colorizer
+from .logutils import setup_loghandlers
 from .exceptions import NoQueueError, UnpickleError
 from .timeouts import death_penalty_after
 from .version import VERSION
@@ -29,6 +26,9 @@ green = make_colorizer('darkgreen')
 yellow = make_colorizer('darkyellow')
 blue = make_colorizer('darkblue')
 
+DEFAULT_RESULT_TTL = 500
+logger = logging.getLogger(__name__)
+
 
 class StopRequested(Exception):
     pass
@@ -96,7 +96,7 @@ class Worker(object):
         return worker
 
 
-    def __init__(self, queues, name=None, default_result_ttl=500,
+    def __init__(self, queues, name=None, default_result_ttl=DEFAULT_RESULT_TTL,
             connection=None, exc_handler=None):  # noqa
         if connection is None:
             connection = get_current_connection()
@@ -112,7 +112,7 @@ class Worker(object):
         self._is_horse = False
         self._horse_pid = 0
         self._stopped = False
-        self.log = Logger('worker')
+        self.log = logger
         self.failed_queue = get_failed_queue(connection=self.connection)
 
         # By default, push the "move-to-failed-queue" exception handler onto
@@ -282,6 +282,7 @@ class Worker(object):
 
         The return value indicates whether any jobs were processed.
         """
+        setup_loghandlers()
         self._install_signal_handlers()
 
         did_perform_work = False
@@ -308,18 +309,15 @@ class Worker(object):
                 except StopRequested:
                     break
                 except UnpickleError as e:
-                    msg = '*** Ignoring unpickleable data on %s.' % \
-                            green(e.queue.name)
-                    self.log.warning(msg)
-                    self.log.debug('Data follows:')
-                    self.log.debug(e.raw_data)
-                    self.log.debug('End of unreadable data.')
-                    self.failed_queue.push_job_id(e.job_id)
+                    job = Job.safe_fetch(e.job_id)
+                    self.handle_exception(job, *sys.exc_info())
                     continue
 
                 self.state = 'busy'
 
                 job, queue = result
+                # Use the public setter here, to immediately update Redis
+                job.status = Status.STARTED
                 self.log.info('%s: %s (%s)' % (green(queue.name),
                     blue(job.description), job.id))
 
@@ -372,7 +370,7 @@ class Worker(object):
         signal.signal(signal.SIGTERM, signal.SIG_DFL)
 
         self._is_horse = True
-        self.log = Logger('horse')
+        self.log = logger
 
         success = self.perform_job(job)
 
